import 'package:analyzer/dart/element/element.dart';
import 'package:auto_model/src/annotations.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';
import 'internal_generator_mixin.dart';

class CopyGenerator extends GeneratorForAnnotation<Copy>
    with InternalGeneratorMixin {
  @override
  String generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    final fieldElements = fieldElementsOf(element);
    if (fieldElements.isNotEmpty) {
      final params = fieldElements
          .map((fieldElement) {
            final setter = fieldElement.setter;
            if (setter != null && setter.isSetter) {
              // generic type
              return '${fieldElement.type.displayName} ${fieldElement.name}';
            } else {
              return null;
            }
          })
          .where((ele) => ele != null)
          .join(', ');

      final fieldSetters = fieldElements
          .map((fieldElement) {
            var setter = fieldElement.setter;
            if (setter != null && setter.isSetter) {
              var name = fieldElement.name;
              return '''
                if ($name != null) {
                  copied.$name = $name;
                } else {
                  copied.$name = this.$name;
                }
              ''';
            } else {
              return null;
            }
          })
          .where((ele) => ele != null)
          .join('\n');
      return '''
    extension ${extensionClassName(element, annotation)} on ${element.name} {
      /// Used for copy an object altering some of its properties, 
      /// but keeping the rest unchanged.
      ${element.name} copy({$params}) {
        var copied = ${element.name}(); // ignore: prefer_final_locals
        $fieldSetters
        return copied;
      }
    }
    ''';
    } else {
      throw Exception('');
    }
  }
}
