import 'dart:convert';
import 'dart:mirrors';

import 'package:auto_model/auto_model.dart';

const Symbol _hashCodeSymbol = Symbol('hashCode');
const Symbol _runtimeTypeSymbol = Symbol('runtimeType');

String _nameOf(Symbol symbol) {
  final str = symbol.toString();
  final start = 'Symbol("'.length;
  final end = str.length - '")'.length;
  return str.substring(start, end);
}

Object _getAnnotation(DeclarationMirror declaration, Type annotation) {
  final reflectee = declaration.metadata
      .firstWhere((instanceMirror) => instanceMirror.hasReflectee, orElse: () => null)
      ?.reflectee;
  if (reflectee?.runtimeType == annotation) {
    return reflectee;
  }
  return null;
}



String toJson(Object value) {
  return jsonEncode(value, toEncodable: (nonEncodable) {
    final toMap = <String, dynamic>{};

    final instanceMirror = reflect(nonEncodable);
    instanceMirror.type.instanceMembers.forEach((symbol, mirror) {
      if (mirror.isGetter &&
          symbol != _hashCodeSymbol &&
          symbol != _runtimeTypeSymbol) {
        final annotation = _getAnnotation(
            instanceMirror.type.declarations[symbol], SerializedName);
        String key;
        if (annotation is SerializedName) {
          key = annotation.value;
        } else {
          key = _nameOf(symbol);
        }
        toMap[key] = instanceMirror
            .getField(symbol)
            .reflectee;
      }
    });
    return toMap;
  });
}
