import 'package:analyzer/dart/element/element.dart';
import 'package:auto_model/src/annotations.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

import 'internal_generator_mixin.dart';

class ToStringGenerator extends GeneratorForAnnotation<ToString>
    with InternalGeneratorMixin {
  @override
  String generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    final fieldElements = fieldElementsOf(element);
    if (fieldElements.isNotEmpty) {
      final toString = fieldElements
          .map((fieldElement) => '${fieldElement.name}=\$${fieldElement.name}')
          .join(', ');
      return '''
    extension ${extensionClassName(element, annotation)} on ${element.name} {
      /// Used for generate a string with all available fields form like 
      /// "User(name=John, age=42)".
      String toString2() {
        return '${element.name}($toString)';
      }
    }
    ''';
    } else {
      throw Exception('');
    }
  }
}
